@@ -1,12 +1,12 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * gmpy_convert_mpfr.c                                                     *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * Python interface to the GMP or MPIR, MPFR, and MPC multiple precision   *
+ * Python interface to the GMP, MPFR, and MPC multiple precision           *
  * libraries.                                                              *
  *                                                                         *
  * Copyright 2000 - 2009 Alex Martelli                                     *
  *                                                                         *
- * Copyright 2008 - 2021 Case Van Horsen                                   *
+ * Copyright 2008 - 2024 Case Van Horsen                                   *
  *                                                                         *
  * This file is part of GMPY2.                                             *
  *                                                                         *
@@ -117,7 +117,7 @@ GMPy_MPFR_From_MPFR(MPFR_Object *obj, mpfr_prec_t prec, CTXT_Object *context)
 }
 
 static MPFR_Object *
-GMPy_MPFR_From_PyIntOrLong(PyObject *obj, mpfr_prec_t prec, CTXT_Object *context)
+GMPy_MPFR_From_PyLong(PyObject *obj, mpfr_prec_t prec, CTXT_Object *context)
 {
     MPFR_Object *result = NULL;
     MPZ_Object *tempx = NULL;
@@ -140,7 +140,7 @@ GMPy_MPFR_From_PyIntOrLong(PyObject *obj, mpfr_prec_t prec, CTXT_Object *context
 
     if ((temp == -1) && PyErr_Occurred()) {
         PyErr_Clear();
-        if (!(tempx = GMPy_MPZ_From_PyIntOrLong(obj, context))) {
+        if (!(tempx = GMPy_MPZ_From_PyLong(obj, context))) {
             return NULL;
         }
         if (was_one)
@@ -293,7 +293,7 @@ GMPy_MPFR_From_PyStr(PyObject *s, int base, mpfr_prec_t prec, CTXT_Object *conte
     MPQ_Object *tempq;
     char *cp, *endptr;
     Py_ssize_t len;
-    PyObject *ascii_str = ascii_str = GMPy_RemoveUnderscoreASCII(s);
+    PyObject *ascii_str = ascii_str = GMPy_RemoveIgnoredASCII(s);
 
     if (!ascii_str) return NULL;
 
@@ -342,7 +342,7 @@ GMPy_MPFR_From_PyStr(PyObject *s, int base, mpfr_prec_t prec, CTXT_Object *conte
     }
 
     /* If the context requests subnormals and the result is in the range for subnormals,
-     * we use exact conversion via conversion to an mpq.
+     * we use exact conversion via intermediate conversion to an mpq.
      *
      * The sticky bit returned by MPFR's string conversion appears to only reflect the
      * portion of the string needed to compute the correctly rounded result. It does not
@@ -352,7 +352,8 @@ GMPy_MPFR_From_PyStr(PyObject *s, int base, mpfr_prec_t prec, CTXT_Object *conte
      * sticky bit.
      */
 
-    if (base == 10 &&
+    if (mpfr_regular_p(result->f) &&
+        base == 10 &&
         context->ctx.subnormalize &&
         result->f->_mpfr_exp <= context->ctx.emin + mpfr_get_prec(result->f) - 1)
         {
@@ -406,7 +407,7 @@ GMPy_MPFR_From_RealWithType(PyObject *obj, int xtype, mp_prec_t prec, CTXT_Objec
         return GMPy_MPFR_From_MPZ((MPZ_Object*)obj, prec, context);
 
     if (IS_TYPE_PyInteger(xtype))
-        return GMPy_MPFR_From_PyIntOrLong(obj, prec, context);
+        return GMPy_MPFR_From_PyLong(obj, prec, context);
 
     if (IS_TYPE_PyFraction(xtype))
         return GMPy_MPFR_From_Fraction(obj, prec, context);
@@ -763,29 +764,6 @@ GMPy_MPQ_From_MPFR(MPFR_Object *self, CTXT_Object *context)
     return result;
 }
 
-static PyObject *
-GMPy_PyIntOrLong_From_MPFR(MPFR_Object *obj, CTXT_Object *context)
-{
-    PyObject *result;
-    MPZ_Object *tempz;
-
-    CHECK_CONTEXT(context);
-
-    if (!(tempz = GMPy_MPZ_From_MPFR(obj, context)))
-        return NULL;
-
-    result = GMPy_PyIntOrLong_From_MPZ(tempz, context);
-    Py_DECREF((PyObject*)tempz);
-
-    return result;
-}
-
-static PyObject *
-GMPy_MPFR_Int_Slot(MPFR_Object *self) {
-    return GMPy_PyIntOrLong_From_MPFR(self, NULL);
-}
-
-#ifdef PY2
 static PyObject *
 GMPy_PyLong_From_MPFR(MPFR_Object *obj, CTXT_Object *context)
 {
@@ -804,10 +782,9 @@ GMPy_PyLong_From_MPFR(MPFR_Object *obj, CTXT_Object *context)
 }
 
 static PyObject *
-GMPy_MPFR_Long_Slot(MPFR_Object *self) {
+GMPy_MPFR_Int_Slot(MPFR_Object *self) {
     return GMPy_PyLong_From_MPFR(self, NULL);
 }
-#endif
 
 static PyObject *
 GMPy_PyFloat_From_MPFR(MPFR_Object *self, CTXT_Object *context)
@@ -911,7 +888,7 @@ GMPy_MPFR_Str_Slot(MPFR_Object *self)
 
     sprintf(fmtstr, "{0:.%ldg}", precision);
 
-    temp = Py_BuildValue("s", fmtstr);
+    temp = PyUnicode_FromString(fmtstr);
     if (!temp)
         return NULL;
     result = PyObject_CallMethod(temp, "format", "O", self);
@@ -934,7 +911,7 @@ GMPy_MPFR_Repr_Slot(MPFR_Object *self)
     else
         sprintf(fmtstr, "mpfr('{0:.%ldg}')", precision);
 
-    temp = Py_BuildValue("s", fmtstr);
+    temp = PyUnicode_FromString(fmtstr);
     if (!temp)
         return NULL;
     result = PyObject_CallMethod(temp, "format", "O", self);
